@@ -27,6 +27,8 @@ import pandas as pd  # type: ignore
 from ..environment import AbstractEnvironment
 from ..activelearning.base import ActiveLearner
 from ..activelearning.estimator import Estimator
+from ..activelearning.target import TargetMethod
+from ..activelearning.autotarensemble import AutoTARFirstMethod
 
 from ..typehints import KT, LT, IT, DT, VT, RT
 
@@ -76,7 +78,7 @@ class RandomInitializer(Initializer[IT, KT, LT], Generic[IT, KT, LT]):
             itertools.chain.from_iterable(
                 map(
                     lambda lbl: self.get_random_sample_for_label(learner, lbl),
-                    learner.env.labels.labelset,
+                    sorted(learner.env.labels.labelset),
                 )
             )
         )
@@ -107,6 +109,58 @@ class RandomInitializer(Initializer[IT, KT, LT], Generic[IT, KT, LT]):
         return builder_func
 
 
+class SeededRandomInitializer(RandomInitializer[IT, KT, LT], Generic[IT, KT, LT]):
+    def __init__(self, sample_size: int = 1, seed: int = 42) -> None:
+        super().__init__(sample_size)
+        self.seed = seed
+        self.rng = np.random.default_rng(seed)
+
+    def get_random_sample_for_label(
+        self, learner: ActiveLearner[IT, KT, Any, Any, Any, LT], label: LT
+    ) -> Sequence[KT]:
+        docs = self.rng.choice(
+            list(learner.env.truth.get_instances_by_label(label)), self.sample_size  # type: ignore
+        )
+        return docs
+
+    @classmethod
+    def builder(
+        cls, sample_size: int, seed: int, *args, **kwargs
+    ) -> Callable[..., Self]:
+        def builder_func(*args, **kwargs) -> Self:
+            return cls(sample_size, seed)
+
+        return builder_func
+
+
+class TargetInitializer(RandomInitializer[IT, KT, LT], Generic[IT, KT, LT]):
+    def __call__(
+        self, learner: ActiveLearner[IT, KT, DT, VT, RT, LT]
+    ) -> ActiveLearner[IT, KT, DT, VT, RT, LT]:
+        if not isinstance(learner, TargetMethod):
+            return super().__call__(learner)
+        sublearner = learner.learners[1]
+        docs = self.get_initialization_sample(learner)
+        for doc in docs:
+            self.add_doc(sublearner, doc)
+            self.add_doc(learner, doc)
+        return learner
+
+
+class PriorInitializer(RandomInitializer[IT, KT, LT], Generic[IT, KT, LT]):
+    def __call__(
+        self, learner: ActiveLearner[IT, KT, DT, VT, RT, LT]
+    ) -> ActiveLearner[IT, KT, DT, VT, RT, LT]:
+        if not isinstance(learner, AutoTARFirstMethod):
+            return super().__call__(learner)
+        sublearner = learner.learners[0]
+        docs = self.get_initialization_sample(learner)
+        for doc in docs:
+            self.add_doc(sublearner, doc)
+            self.add_doc(learner, doc)
+        return learner
+
+
 class UniformInitializer(RandomInitializer[IT, KT, LT], Generic[IT, KT, LT]):
     def __call__(
         self, learner: ActiveLearner[IT, KT, DT, VT, RT, LT]
