@@ -1,22 +1,17 @@
 from __future__ import annotations
 
 import collections
-import itertools
 import logging
 import os
-from abc import ABC, abstractmethod
 from typing import (
     Any,
     Deque,
     Dict,
     FrozenSet,
     Generic,
-    Iterable,
-    List,
     Optional,
     Sequence,
     Tuple,
-    TypeVar,
 )
 import warnings
 
@@ -24,13 +19,11 @@ import numpy as np
 import numpy.typing as npt
 
 import pandas as pd  # type: ignore
-
+from ..activelearning.autotarensemble import AutoTARFirstMethod
 from ..activelearning.base import ActiveLearner
 from ..activelearning.estimator import Estimator
 from ..utils.func import (
-    all_subsets,
     intersection,
-    list_unzip3,
     not_in_supersets,
     powerset,
     union,
@@ -76,12 +69,17 @@ class ChaoEstimator(
         self.matrix_history: Deque[pd.DataFrame] = collections.deque()
         self.contingency_history: Deque[Dict[FrozenSet[int], int]] = collections.deque()
         self.r_loaded = False
+        self.df: Optional[pd.DataFrame] = None
+        self.est = Estimate.empty()
+        self.rfile = "mhmodel.R"
+        self.rfunc = "get_abundance"
+        self.it = 0
 
     def _start_r(self) -> None:
         _check_R()
         R = ro.r
         filedir = os.path.dirname(os.path.realpath(__file__))
-        r_script_file = os.path.join(filedir, "mhmodel.R")
+        r_script_file = os.path.join(filedir, self.rfile)
         R["source"](r_script_file)
 
     def get_label_matrix(
@@ -136,47 +134,60 @@ class ChaoEstimator(
                     matrix[i, j] = len(intersection)
         return matrix
 
-    def calculate_abundance_R(
-        self, estimator: Estimator[Any, KT, DT, VT, RT, LT], label: LT
-    ) -> pd.DataFrame:
+    def calculate_abundance_R(self, df: pd.DataFrame) -> pd.DataFrame:
         if not self.r_loaded:
             self._start_r()
             self.r_loaded = True
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            df = self.get_label_matrix(estimator, label)
             with localconverter(ro.default_converter + pandas2ri.converter):
                 df_r = ro.conversion.py2rpy(df)
-                abundance_r = ro.globalenv["get_abundance"]
+                abundance_r = ro.globalenv[self.rfunc]
                 r_df = abundance_r(df_r)
                 res_df: pd.DataFrame = ro.conversion.rpy2py(r_df)
+                if self.it % 100 == 0:
+                    ro.r("gc()")
+                self.it += 1
         return res_df
 
     def calculate_abundance(
         self, estimator: Estimator[Any, KT, DT, VT, RT, LT], label: LT
     ) -> Estimate:
-        res_df = self.calculate_abundance_R(estimator, label)
+        df = self.get_label_matrix(estimator, label)
+        old_df = self.df
+        if old_df is None or not old_df.equals(df):
+            self.df = df
+            res_df = self.calculate_abundance_R(df)
 
-        def try_float(val: Any) -> float:
-            try:
-                parsed = float(val)
-            except ValueError:
-                parsed = float("nan")
-            return parsed
+            def try_float(val: Any) -> float:
+                try:
+                    parsed = float(val)
+                except:
+                    parsed = float("nan")
+                return parsed
 
-        point = try_float(res_df["abundance"][0])
-        lower = try_float(res_df["infCL"][0])
-        upper = try_float(res_df["supCL"][0])
-        return Estimate(point, lower, upper)
+            point = try_float(res_df["abundance"][0])
+            lower = try_float(res_df["infCL"][0])
+            upper = try_float(res_df["supCL"][0])
+            self.est = Estimate(point, lower, upper)
+        return self.est
 
     def __call__(
         self, learner: ActiveLearner[Any, KT, DT, VT, RT, LT], label: LT
     ) -> Estimate:
         empty = np.array([])
         stats = EstimationModelStatistics(empty, empty, 0.0, empty)
-        if not isinstance(learner, Estimator):
-            return Estimate(0.0, 0.0, 0.0)
-        estimate = self.calculate_abundance(learner, label)
+        if not isinstance(learner, (Estimator, AutoTARFirstMethod)):
+            return Estimate(float("nan"), float("nan"), float("nan"))
+        if isinstance(learner, AutoTARFirstMethod):
+            l2 = learner.learners[2]
+            assert isinstance(l2, Estimator)
+            if l2.env.labeled:
+                estimate = self.calculate_abundance(l2, label)
+            else:
+                estimate = Estimate(float("nan"), float("nan"), float("nan"))
+        else:
+            estimate = self.calculate_abundance(learner, label)
         return estimate
 
     def all_estimations(
@@ -233,18 +244,12 @@ class ChaoEstimator(
 class ChaoAlternative(
     ChaoEstimator[IT, KT, DT, VT, RT, LT], Generic[IT, KT, DT, VT, RT, LT]
 ):
-    def calculate_abundance_R(
-        self, estimator: Estimator[Any, KT, DT, VT, RT, LT], label: LT
-    ) -> pd.DataFrame:
-        if not self.r_loaded:
-            self._start_r()
-            self.r_loaded = True
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            df = self.get_label_matrix(estimator, label)
-            with localconverter(ro.default_converter + pandas2ri.converter):
-                df_r = ro.conversion.py2rpy(df)
-                abundance_r = ro.globalenv["get_abundance_eta"]
-                r_df = abundance_r(df_r)
-                res_df: pd.DataFrame = ro.conversion.rpy2py(r_df)
-        return res_df
+    def __init__(self):
+        super().__init__()
+        self.rfunc = "get_abundance_eta"
+
+
+class LogLinear(ChaoEstimator[IT, KT, DT, VT, RT, LT], Generic[IT, KT, DT, VT, RT, LT]):
+    def __init__(self):
+        super().__init__()
+        self.rfunc = "get_abundance_ll"
